/****************************************************************************
** COPYRIGHT (C):    1998 M. Schovanek, M. Kosturik
** PROJEKT:          OOP
** SOUBOR:           graf.cpp
** UCEL:             Implementace grafu.
** VERZE             1.0
** JAZYK:            bcc
** CILOVY POCITAC:   IBM PC
** PROGRAMATOR:      Martin Schovanek (SCH)
** HISTORIE UPRAV:
**
****************************************************************************/
/* POZNAMKY ****************************************************************

****************************************************************************/

#define TEST

/* HLAVICKY ****************************************************************/
#include "graf.h"
#include <values.h>
/*-------------------------------------------------------------------------*/

Sch_Graf::Sch_Graf()
/* UCEL:     Konstruktor Grafu. Inicializuje Graf.
   VSTUP:
   VRACI:
   POZNAMKY:
*/
:  _num_node(0),
   _num_edge(0),
   _prec(5.0)
{  // tu by se mohla mazat pole _nodes a _edges
}

/*-------------------------------------------------------------------------*/

Sch_Graf::~Sch_Graf()
/* UCEL:     Destruktor Grafu. Likviduje Graf.
   VSTUP:
   VRACI:
   POZNAMKY:
*/
{  del_graf();
}



//...........................................................................

void Sch_Graf::plot(Sch_GuiBgi& gui) const
/* UCEL:     Vykresli cely graf na dane gui.
   VSTUP:    Odkaz na gui.
   VRACI:
   POZNAMKY:
*/
{  for (int i=0; i < _num_node; i++) _nodes[i]->plot(gui);
   for (i=0; i < _num_edge; i++) _edges[i]->plot(gui);
}

//...........................................................................

Bool Sch_Graf::add_node(double x, double y, char* name)
/* UCEL:     Prida do grafu vrchol.
   VSTUP:    x,y - souradnice stredu.
             name - jmeno vrcholu.
   VRACI:    TRUE - pokud se to povedlo jinak FALSE.
   POZNAMKY:
*/
{  if (_num_node >= MAXNODE - 1) return FALSE;
   if (!(_nodes[_num_node] = new Sch_Node(x, y, name))) return FALSE;
   _num_node++;
   return TRUE;
}

//...........................................................................

Bool Sch_Graf::add_edge(int start, int final, char* name)
/* UCEL:     Prida do grafu hranu.
   VSTUP:    start, final - pocatecni a koncovy vrchol.
             name - jmeno hrany.
   VRACI:    TRUE - pokud se to povedlo jinak FALSE.
   POZNAMKY:
*/
{  if (_num_edge >= MAXEDGE - 1) return FALSE;
   if (!(_edges[_num_edge] = new Sch_Edge(_nodes[start], _nodes[final], name))) return FALSE;
   _num_edge++;
   return TRUE;
}

//...........................................................................

Bool Sch_Graf::del_node(int node)
/* UCEL:     Odstrani z grafu vrchol .
   VSTUP:    node - rukojet vrcholu.
   VRACI:    TRUE - pokud se to povedlo jinak FALSE.
   POZNAMKY:
*/
{  // 1. Overit platnost node.
   if (node > _num_node) return FALSE;

   // 2. Projit hrany a pokud inciduji s vrcholem tak smazat.
   for (int i=0; i < _num_edge; i++)
      if ((_edges[i]->start() == _nodes[node]) ||
          (_edges[i]->final() == _nodes[node])) del_edge(i--);

   // 3. Odstranit vrchol + posunout pole, _num_node--.
   delete _nodes[node];
   for (i=node; i < _num_node; i++) _nodes[i]=_nodes[i+1];
   _num_node--;
   return TRUE;
}

//...........................................................................

Bool Sch_Graf::del_edge(int edge)
/* UCEL:     Odstrani z grafu hranu .
   VSTUP:    edge - rukojet hrany.
   VRACI:    TRUE - pokud se to povedlo jinak FALSE.
   POZNAMKY:
*/
{  // 1. Overit platnost edge.
   if (edge > _num_edge) return FALSE;

   // 2. Odstranit vrchol + posunout pole, _num_edge--.
   delete _edges[edge];
   for (int i=edge; i < _num_edge; i++) _edges[i]=_edges[i+1];
   _num_edge--;
   return TRUE;
}

//...........................................................................

void Sch_Graf::ch_node_name(int node, char* name)
/* UCEL:     Zmeni jmeno vrcholu .
   VSTUP:    Poradi vrcholu v poli vrcholu, nove jmeno.
   VRACI:
   POZNAMKY:
*/
{
   if (node > _num_node) return;

   _nodes[node]->change_name(name);
   return;
}
//...........................................................................

void Sch_Graf::ch_edge_name(int edge, char* name)
/* UCEL:     Zmeni jmeno hrany .
   VSTUP:    Poradi hrany v poli hran, nove jmeno.
   VRACI:
   POZNAMKY:
*/
{
   if (edge > _num_edge) return;

   _edges[edge]->change_name(name);
   return;
}
//...........................................................................

Bool Sch_Graf::move_node(int node, double x, double y)
/* UCEL:     Zmeni pozici vrcholu .
   VSTUP:    Poradi vrcholu v poli vrchlolu, nove souradnice .
   VRACI:    TRUE - pokud doslo k posunuti vrcholu jinak,
             FALSE.
   POZNAMKY:
*/
{
   if (node > _num_node) return FALSE;
   if ((_nodes[node]->x() != x) || (_nodes[node]->y() != y))
   {  _nodes[node]->move(x, y);
      return TRUE;
   }
   else return FALSE;
}

//...........................................................................

int Sch_Graf::find_node(double x, double y)
/* UCEL:     Hleda vrchol na ktery se kliknulo mysi .
   VSTUP:    souradnice kliknuti mysi .
   VRACI:    Pokud najde vrati poradove cislo vrcholu v poli vrchlolu,
             jinak vraci -1.
   POZNAMKY:
*/
{
   for (int i = 0; i < _num_node; i++)
      if (fabs(_nodes[i]->x() - x) < _prec &&
          fabs(_nodes[i]->y() - y) < _prec   ) return i;
   return -1;
}

//...........................................................................

int Sch_Graf::find_edge(double x, double y)
/* UCEL:     Hleda hranu na kterou se kliknulo mysi .
   VSTUP:    Souradnice kliknuti mysi .
   VRACI:    Pokud najde vrati poradove cislo hrany v poli hran,
             jinak vraci -1.
   POZNAMKY:
*/
{
   double a, b ,c, l, lmin=MAXDOUBLE;
   int min_edge = -1;
   const Sch_Node* A;
   const Sch_Node* B;


   for (int i = 0; i < _num_edge; i++)
   {

      A = _edges[i]->start();
      B = _edges[i]->final();
      a = B->y() - A->y();
      b = A->x() - B->x();
      c =-A->x() * B->y() + A->y() * B->x();
      if ((a == .0) && (b == .0)) continue;
      l = fabs(a*x + b*y +c) / sqrt(a*a + b*b);

      if ((l < _prec) && (l < lmin)) lmin=l, min_edge = i;

   }
   return(min_edge);
}

//...........................................................................

void Sch_Graf::del_graf()
/* UCEL:     Smaze graf.
   VSTUP:
   VRACI:
   POZNAMKY:
*/
{  for (int i=0; i < _num_edge; i++) delete _edges[i];
   for (i=0; i < _num_node; i++) delete _nodes[i];
}

//...........................................................................

Bool Sch_Graf::load(char& name)
/* UCEL:     Nacte graf ze souboru.
   VSTUP:    Jmeno souboru.
   VRACI:    TRUE - pokud operace probehla uspesne jinak FALSE.
   POZNAMKY:
*/
{  return TRUE;
}

//...........................................................................

Bool Sch_Graf::save(char& name)
/* UCEL:     Ulozi graf do souboru.
   VSTUP:    Jmeno souboru.
   VRACI:    TRUE - pokud operace probehla uspesne jinak FALSE.
   POZNAMKY:
*/
{  return TRUE;
}

//...........................................................................

void Sch_Graf::select_node(int node)
/* UCEL:     Nastavi nebo schodi vlajku _select.
             Tj. jestli je vrchol oznaceny.
   VSTUP:    Index vrcholu.
   VRACI:
   POZNAMKY:
*/
{
   _nodes[node]->select();

   return;
}

//...........................................................................

void Sch_Graf::select_edge(int edge)
/* UCEL:     Nastavi nebo schodi vlajku _select.
             Tj. jestli je hrana oznacena.
   VSTUP:    Index hrany.
   VRACI:
   POZNAMKY:
*/
{
   _edges[edge]->select();

   return;
}


