/*****************************************************************************
** COPYRIGHT (C):    1998 M. Schovanek, M. Kosturik
** PROJEKT:          OOP
** SOUBOR:           gui_bgi.cpp
** UCEL:             Graficke rozhrani pro Borland-BGI 
** VERZE             1.0
** JAZYK:            bcc
** CILOVY POCITAC:   IBM PC
** PROGRAMATOR:      Martin Schovanek (SCH)
** HISTORIE UPRAV:
**
******************************************************************************/
/* POZNAMKY ******************************************************************

******************************************************************************/



/* HLAVICKY ******************************************************************/

#include "gui_bgi.h"

/* FUNCTIONS *****************************************************************/

Sch_GuiBgi::Sch_GuiBgi()
/* UCEL:     Konstruktor GUI. Inicializuje GUI a obrazovku
   VSTUP:
   VRACI:
   POZNAMKY:
*/
:  _font_size(0),
   _font(DEFAULT_FONT)

{  const double DEF_XMIN = 0;
   const double DEF_YMIN = 0;
   const double DEF_XMAX = 639;
   const double DEF_YMAX = 479;

   int gdriver = DETECT;
   int gmode;
   initgraph(&gdriver, &gmode, "");
   int errorcode = graphresult();
   if (errorcode != grOk)
   {  cerr << "bgi_init: Nelze inicializovat grafiku. Zkontrolujte .BGI soubor."
	 << endl;
      exit(1);
   }

   _d_xmax = getmaxx();
   _d_ymax = getmaxy();
   set_window_coord(DEF_XMIN, DEF_YMIN, DEF_XMAX, DEF_YMAX);
   set_font_style(DEFAULT_FONT);
   set_pen_style(DEFAULT_PEN);
   set_pen_color(DEFAULT_COLOR);
}

//............................................................................

void Sch_GuiBgi::close(void)
/* UCEL:     Ukoncuje cinnost GUI.
   VSTUP:
   VRACI:
   POZNAMKY:
*/
{
   closegraph();
}

//............................................................................

void Sch_GuiBgi::set_window_coord(double xmin, double ymin,
      double xmax, double ymax)
/* UCEL:     Nastavuje uzivatelske souradnice obrazovky
   VSTUP:    xmin, ymin - souradnice leveho horniho rohu.
	          xmax, ymax - souradnice praveho dolniho rohu.
   VRACI:
   POZNAMKY:
*/
{  _u_xmin = xmin;
   _u_xmax = xmax;
   _u_ymin = ymin;
   _u_ymax = ymax;
}

//............................................................................

void Sch_GuiBgi::circ(double x, double y,
   double radius)
/* UCEL:     Zobrazi kruznici se stredem v x,y o polomeru radius.
   VSTUP:    x, y - souradnice stredu.
	          radius - polomer kruznice.
   VRACI:
   POZNAMKY:
*/
{  circle(u2d_x(x), u2d_y(y), u2d_dx(radius));
}

//............................................................................

void Sch_GuiBgi::plot(double x1, double y1,
   double x2, double y2)
/* UCEL:     Nakresli usecku z bodu [x1,y1] do bodu [x2,y2].
   VSTUP:    x1, y1, x2, y2 - souradnice bodu.
   VRACI:
   POZNAMKY:
*/
{  line(u2d_x(x1), u2d_y(y1), u2d_x(x2), u2d_y(y2));
}

//............................................................................

void Sch_GuiBgi::rectangle(double xleft, double ytop, double xright, double ybot)
/* UCEL:     Kresli obdelnik.
   VSTUP:    xleft, ytop - levy horni roh
             xright, ybottom - dolni pravy roh
   VRACI:
   POZNAMKY:
*/
{  int x1 = u2d_x(xleft);
   int x2 = u2d_x(xright);
   int y1 = u2d_y(ytop);
   int y2 = u2d_y(ybot);
   if (x1 > x2) sch_swap(x1, x2);
   if (y1 > y2) sch_swap(y1, y2);

   ::rectangle(x1, y1, x2, y2);
}

//............................................................................

void Sch_GuiBgi::bar(double xleft, double ytop, double xright, double ybot,
                     enum Color color)
/* UCEL:     Kresli plnou oblast.
   VSTUP:    xleft, ytop - levy horni roh
             xright, ybottom - dolni pravy roh
   VRACI:
   POZNAMKY:
*/
{  int x1 = u2d_x(xleft);
   int x2 = u2d_x(xright);
   int y1 = u2d_y(ytop);
   int y2 = u2d_y(ybot);
   if (x1 > x2) sch_swap(x1, x2);
   if (y1 > y2) sch_swap(y1, y2);

   setfillstyle(SOLID_FILL, color);
   ::bar(x1, y1, x2, y2);
}

//............................................................................

void Sch_GuiBgi::text(char* t, double x, double y)
/* UCEL:     Zobrazi text.
   VSTUP:    t - text
             x, y - levy horni roh textu
   VRACI:
   POZNAMKY:
*/
{  int dx = u2d_x(x);
   int dy = u2d_y(y);
   if (_font != SYS_FONT) dy -= textheight("X") / 4;
   outtextxy(dx, dy, t);
}

//............................................................................

int Sch_GuiBgi::u2d_x(double x) const
/* UCEL:     Prepocitava uzivatelske souradnice na souradnice obrazovky.
   VSTUP:    Uziv. sour. X.
   VRACI:    Prepoctena souradnice.
   POZNAMKY:
*/
{  return (int)((x - _u_xmin) * _d_xmax / (_u_xmax - _u_xmin));
}

//............................................................................

int Sch_GuiBgi::u2d_y(double y) const
/* UCEL:     Prepocitava uzivatelske souradnice na souradnice obrazovky.
   VSTUP:    Uziv. sour. Y.
   VRACI:    Prepoctena souradnice.
   POZNAMKY:
*/
{  return (int)((y - _u_ymin) * _d_ymax / (_u_ymax - _u_ymin));
}

//............................................................................

int Sch_GuiBgi::u2d_dx(double x) const
/* UCEL:     Prepocitava uzivatelske diference (delky) na diference obrazovky.
   VSTUP:    Uziv. dif. X.
   VRACI:    Prepoctena diference.
   POZNAMKY: Pouziti napr.: pokud mame kreslit kruznici a polomer je zadany
             v uziv. souradnicich.
*/
{  return (int)(x * _d_xmax / (_u_xmax - _u_xmin));
}

//............................................................................

int Sch_GuiBgi::u2d_dy(double y) const
/* UCEL:     Prepocitava uzivatelske diference (delky) na diference obrazovky.
   VSTUP:    Uziv. dif. Y.
   VRACI:    Prepoctena diference.
   POZNAMKY: Pouziti napr.: pokud mame kreslit kruznici a polomer je zadany
             v uziv. souradnicich.
*/
{  return (int)(y * _d_ymax / (_u_ymax - _u_ymin));
}

//............................................................................

double Sch_GuiBgi::d2u_x(int x) const
/* UCEL:     Prepocitava souradnice obrazovky na uzivatelske souradnice.
   VSTUP:    Sour. obrazovky X.
   VRACI:    Prepoctena souradnice.
   POZNAMKY:
*/
{  return (double)x * (_u_xmax - _u_xmin) / _d_xmax;
}

//............................................................................

double Sch_GuiBgi::d2u_y(int y) const
/* UCEL:     Prepocitava souradnice obrazovky na uzivatelske souradnice.
   VSTUP:    Sour. obrazovky Y.
   VRACI:    Prepoctena souradnice.
   POZNAMKY:
*/
{  return (double)y * (_u_ymax - _u_ymin) / _d_ymax;
}

//............................................................................

void Sch_GuiBgi::set_pen_style(PenStyle s)
/* PURPOSE:    set the pen thickness
   RECEIVES:   t - the pen thickness
*/
{  static int gc2bgi[] =
   {   ::SOLID_LINE, ::SOLID_LINE, ::DOTTED_LINE, ::DASHED_LINE
      /* NORMAL_PEN, THICK_PEN, DOTTED_PEN, DASHED_PEN */
   };

   struct linesettingstype settings;
   ::getlinesettings(&settings);
   ::setlinestyle(gc2bgi[s], settings.upattern,
      s == THICK_PEN ? ::THICK_WIDTH : ::NORM_WIDTH);
}

//............................................................................

void Sch_GuiBgi::set_pen_color(Color col)
/* PURPOSE:   set the color for writing operations
   RECEIVES:  col - the color (BLACK TO WHITE)
*/
{  ::setcolor(col);
}

//............................................................................


void Sch_GuiBgi::set_font_style(FontStyle style)
/* PURPOSE:    set the font style for text
   RECEIVES:   style - the font style
*/
{  if (_font == style) return;

   static int gc2bgi[] =
      { DEFAULT_FONT, SANS_SERIF_FONT, TRIPLEX_FONT };
      /* SYS_FONT, SANS_FONT, ROMAN_FONT */
   textsettingstype settings;
   gettextsettings(&settings);
   settextstyle(gc2bgi[style], settings.direction, style == SYS_FONT ? 1 : 0);
   _font = style;
   set_font_size(_font_size);
}

//............................................................................

void Sch_GuiBgi::set_font_size(double s)
/* PURPOSE:    set the font size for text output
   RECEIVES:   s - the font size (1.0 = normal)
   REMARKS:    Rescale to take points
*/
{  if (_font_size == s) return;
   if (_font != SYS_FONT)
   {  setusercharsize(1, 1, 1, 1);
      double ysize = d2u_y(textheight("X"));

      setusercharsize((int)(1.00 * s / ysize), 1.00, (int)(1.00 * s / ysize), 1.00);
   }
   _font_size = s;
}

//............................................................................

void Sch_GuiBgi::set_clip_on(double x1, double y1, double x2, double y2)
/* UCEL:     Nastavi orezavaci oblast.
   VSTUP:    x1, y1 - levy horni roh
             x2, y2 - pravy dolni roh
   VRACI:
   POZNAMKY:
*/
{
   int dx1 = u2d_x(x1);
   int dx2 = u2d_x(x2);
   int dy1 = u2d_y(y1);
   int dy2 = u2d_y(y2);
   if (dx1 > dx2) sch_swap(dx1, dx2);
   if (dy1 > dy2) sch_swap(dy1, dy2);

   setviewport(dx1, dy1, dx2, dy2, TRUE);
}

//............................................................................

void Sch_GuiBgi::set_clip_off()
/* UCEL:     Vypne orezavani.
   VSTUP:
   VRACI:
   POZNAMKY:
*/
{
   setviewport(0, 0, 0, 0, FALSE);
}

#ifdef TEST

#include <conio.h>

int test()
{  Sch_GuiBgi gui;
   gui.circ(5, 5, 3);
   gui.plot(5, 5, 10, 10);
   getch();
   gui.close();
   return 0;
}


int main()
{  return test();
}
#udef TEST
#endif